/*******************************************************************

Part of the Fritzing project - http://fritzing.org
Copyright (c) 2007-2009 Fachhochschule Potsdam - http://fh-potsdam.de

Fritzing is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fritzing is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.

********************************************************************

$Revision: 3189 $:
$Author: cohen@irascible.com $:
$Date: 2009-06-23 17:27:25 +0000 (Tue, 23 Jun 2009) $

********************************************************************/

#ifndef GROUPITEMBASE_H
#define GROUPITEMBASE_H

#include <QGraphicsItemGroup>
#include <QVariant>
#include <QPainter>

#include "../items/itembase.h"

class GroupItemBase : public ItemBase
{
	Q_OBJECT

public:
	GroupItemBase(class ModelPart* modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu);

	void addToGroup(ItemBase *);
	virtual void syncKinMoved(GroupItemBase *, QPointF newPos);
	virtual void doneAdding(const LayerHash &);

	void findConnectorsUnder();
	void saveGeometry();
	bool itemMoved();
	void saveInstanceLocation(QXmlStreamWriter &);
	void moveItem(ViewGeometry &);
	QRectF boundingRect() const;
	void collectWireConnectees(QSet<class Wire *> & wires);
	void collectFemaleConnectees(QSet<ItemBase *> & items);
	void collectExternalConnectorItems();
	void collectConnectors(QList<ConnectorItem *> & connectors);
	virtual bool isLowerLayerVisible(GroupItemBase *) = 0;
	void setBoundingRect(const QRectF &);
	const QRectF & calcBoundingRect();
	const QRectF & itemsBoundingRect();
	bool hasExternalConnectorItems();
	bool hasConnectors();

protected:
	void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
	void addToGroup(ItemBase *, const LayerHash &);

protected:
	QRectF m_boundingRect;
	QRectF m_itemsBoundingRect;
	QPointF m_offset;
	QList<ItemBase *> m_itemsToAdd;
	QList<class ConnectorItem *> m_externalConnectorItems;
};


#endif
